@@ -7,10 +7,11 @@ use NetPacket::Ethernet;
 use NetPacket::IP;
 use NetPacket::TCP;
 use Net::Pcap; # just for the convenience function below
+use Carp qw(croak);
 
 use vars qw($VERSION);
 
-$VERSION = '0.06';
+$VERSION = '0.07';
 
 =head1 NAME
 
@@ -18,6 +19,7 @@ Sniffer::HTTP - multi-connection sniffer driver
 
 =head1 SYNOPSIS
 
+  use Sniffer::HTTP;
   my $VERBOSE = 0;
 
   my $sniffer = Sniffer::HTTP->new(
@@ -199,28 +201,19 @@ regular expression.
 sub run {
   my ($self,$device_name, $pcap_filter) = @_;
 
+  my $device = $self->find_device($device_name);
   $pcap_filter ||= "port 80";
 
   # Set up Net::Pcap
   my ($err);
   my %devinfo;
-  my @devs = Net::Pcap::findalldevs(\%devinfo, \$err);
-
-  my $device = $device_name;
-  if ($^O eq 'MSWin32') {
-    if (ref $device_name eq 'Regexp') {
-      ($device) = grep {$devinfo{$_} =~ /$device_name/} keys %devinfo;
-    };
+  my @devs;
+  if ($Net::Pcap::VERSION < 0.07) {
+    @devs = Net::Pcap::findalldevs(\$err, \%devinfo);
   } else {
-    $device ||= 'any';
+    @devs = Net::Pcap::findalldevs(\%devinfo, \$err);
   };
 
-  if (! $device) {
-    die "Couldn't find '$device_name' in the devices\n." . Dumper \%devinfo;
-  };
-
-  $self->callbacks->{log}->("Using '$devinfo{$device}'\n");
-
   my ($address, $netmask);
   if (Net::Pcap::lookupnet($device, \$address, \$netmask, \$err)) {
     die 'Unable to look up device information for ', $device, ' - ', $err;
@@ -247,8 +240,141 @@ sub run {
       die 'Unable to perform packet capture';
 };
 
+=head2 C<< $sniffer->find_device DEVICE >>
+
+Finds a L<Net::Pcap> device based on some criteria:
+
+If the parameter given is a regular expression, and
+the operating system is C<MSWin32>, it
+is used to scan the descriptions of the Net::Pcap 
+device list. The name of the first matching element
+is returned.
+
+If a L<Net::Pcap> device matching the
+stringified parameter exists, it is returned.
+If there exists no matching device for the scalar, 
+C<undef> is returned.
+
+If the parameter is not given, and there is a device
+called C<any>, that one is returned.
+
+If there is only one network device, the name of
+that device is returned.
+
+If there is only one device left after removing all 
+network devices with IP address 127.0.0.1, the name
+of that device is returned.
+
+The name of the device with the default gateway
+(if any) is returned.
+
+Otherwise it gives up and returns C<undef>.
+
+=cut
+
+sub find_device {
+  my ($self, $device_name) = @_;
+  # Set up Net::Pcap
+  my ($err);
+  my %devinfo;
+  my @devs;
+  if ($Net::Pcap::VERSION < 0.07) {
+    @devs = Net::Pcap::findalldevs(\$err, \%devinfo);
+  } else {
+    @devs = Net::Pcap::findalldevs(\%devinfo, \$err);
+  };
+
+  my $device = $device_name;
+  if ($device_name) {
+    if (ref $device_name eq 'Regexp') {
+      if ($^O eq 'MSWin32') {
+        ($device) = grep {$devinfo{$_} =~ /$device_name/} keys %devinfo;
+      } else {
+        die "It doesn't make sense to me to scan the devices on $^O.";
+      };
+    } elsif (exists $devinfo{$device_name}) {
+      $device = $device_name;
+    } else {
+      croak "Don't know how to handle $device_name as a Net::Pcap device";
+    };
+  } else {
+    if (exists $devinfo{any}) {
+      $device = 'any';
+    } elsif (@devs == 1) {
+      $device = $devs[0];
+    } else {
+      # Now we need to actually look at the devices and select the
+      # one with the default gateway:
+      
+      # First, get the default gateway by using
+      # `netstat -rn`
+      my $device_ip;
+      my $re_if = $^O eq 'MSWin32' 
+                  ? qr/^\s*(?:0.0.0.0)\s+(\S+)\s+(\S+)\s+/
+                  : qr/^(?:0.0.0.0|default)\s+(\S+)\s+.*?(\S+)\s*$/;
+      for (qx{netstat -rn}) {
+        if ( /$re_if/ ) {
+          #$gateway = $1;
+          $device_ip = $2;
+          last;
+        };
+      };
+      
+      if (! $device_ip) {
+        croak "Couldn't find IP address/interface of the default gateway interface. Maybe 'netstat' is unavailable?";
+      };
+      
+      if (exists $devinfo{$device_ip}) {
+        return $device_ip
+      };
+
+      # Looks like we got an IP and not an interface name.
+      # So scan all interfaces if they have that our IP address.
+      
+      my $good_address = unpack "N", pack "C4", (split /\./, $device_ip);
+      
+      my @good_devices;
+      for my $device (@devs) {
+        my ($address, $netmask);
+        (Net::Pcap::lookupnet($device, \$address, \$netmask, \$err) == 0) or next;
+        $address != 0 or next;
+        for ($address,$netmask) {
+          $_ = unpack "N", pack "N", $_;
+        };
+        
+        if ($address == ($good_address & $netmask)) {
+          push @good_devices, $device;
+        };
+      };
+      
+      if (@good_devices == 1) {
+        $device = $good_devices[0];
+      } elsif (@good_devices > 1) {
+        croak "Too many device candidates found (@good_devices)";
+      }      
+    };
+  };
+  
+  return $device
+};
+
 1;
 
+=head1 EXAMPLE PCAP FILTERS
+
+Here are some example Net::Pcap filters for common things:
+
+Capture all HTTP traffic between your machine and C<www.example.com>:
+
+  (dest www.example.com && (port 80)) || (src www.example.com && (port 80))
+
+Capture all HTTP traffic between your machine and C<www1.example.com> or C<www2.example.com>:
+    (dest www1.example.com && (port 80)) || (src www1.example.com && (port 80))
+  ||(dest www2.example.com && (port 80)) || (src www2.example.com && (port 80))
+
+Note that Net::Pcap resolves these addresses before using them, so you might
+actually get more data than you asked for.
+
 =head1 BUGS
 
 The whole module suite has almost no tests.
